/*******************************************************************************
* File Name: Counter_0_PM.c  
* Version 2.10
*
*  Description:
*     This file provides the power management source code to API for the
*     Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Counter_0.h"

static Counter_0_backupStruct Counter_0_backup;

/*******************************************************************************
* Function Name: Counter_0_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_0_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void Counter_0_SaveConfig(void)
{
    #if (!Counter_0_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Counter_0_PSOC3_ES2 || Counter_0_PSOC5_ES1)
            Counter_0_backup.CounterUdb = Counter_0_ReadCounter();
            Counter_0_backup.CounterPeriod = Counter_0_ReadPeriod();
            Counter_0_backup.CompareValue = Counter_0_ReadCompare();
            Counter_0_backup.InterruptMaskValue = Counter_0_STATUS_MASK;
        #endif
        
        #if (Counter_0_PSOC3_ES3 || Counter_0_PSOC5_ES2)
            Counter_0_backup.CounterUdb = Counter_0_ReadCounter();
            Counter_0_backup.InterruptMaskValue = Counter_0_STATUS_MASK;
        #endif
        
        #if(!Counter_0_ControlRegRemoved)
            Counter_0_backup.CounterControlRegister = Counter_0_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: Counter_0_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_0_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Counter_0_RestoreConfig(void) 
{      
    #if (!Counter_0_UsingFixedFunction)     
        /* Restore the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (Counter_0_PSOC3_ES2 || Counter_0_PSOC5_ES1)
            /* Interrupt State Backup for Critical Region*/
            uint8 Counter_0_interruptState;
        
            Counter_0_WriteCounter(Counter_0_backup.CounterUdb);
            Counter_0_WritePeriod(Counter_0_backup.CounterPeriod);
            Counter_0_WriteCompare(Counter_0_backup.CompareValue);
            /* Enter Critical Region*/
            Counter_0_interruptState = CyEnterCriticalSection();
        
            Counter_0_STATUS_AUX_CTRL |= Counter_0_STATUS_ACTL_INT_EN_MASK;
            /* Exit Critical Region*/
            CyExitCriticalSection(Counter_0_interruptState);
            Counter_0_STATUS_MASK = Counter_0_backup.InterruptMaskValue;
        #endif
        
        #if (Counter_0_PSOC3_ES3 || Counter_0_PSOC5_ES2)
            Counter_0_WriteCounter(Counter_0_backup.CounterUdb);
            Counter_0_STATUS_MASK = Counter_0_backup.InterruptMaskValue;
        #endif
        
        #if(!Counter_0_ControlRegRemoved)
            Counter_0_WriteControlRegister(Counter_0_backup.CounterControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: Counter_0_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_0_backup.enableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*  No
*
*******************************************************************************/
void Counter_0_Sleep(void)
{
    #if(!Counter_0_ControlRegRemoved)
        /* Save Counter's enable state */
        if(Counter_0_CTRL_ENABLE == (Counter_0_CONTROL & Counter_0_CTRL_ENABLE))
        {
            /* Counter is enabled */
            Counter_0_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            Counter_0_backup.CounterEnableState = 0u;
        }
    #endif
    Counter_0_Stop();
    Counter_0_SaveConfig();
}


/*******************************************************************************
* Function Name: Counter_0_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_0_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Counter_0_Wakeup(void) 
{
    Counter_0_RestoreConfig();
    #if(!Counter_0_ControlRegRemoved)
        if(Counter_0_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            Counter_0_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif
    
}


/* [] END OF FILE */
